@@ -1,3 +1,4 @@
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 # Copyright © 2013 Intel Corporation
@@ -50,6 +51,7 @@ class GLProvider(object):
         self.enum = self.enum.replace(' ', '_')
         self.enum = self.enum.replace('\\"', '')
         self.enum = self.enum.replace('.', '_')
+        self.enum = "PROVIDER_" + self.enum
 
 class GLFunction(object):
     def __init__(self, ret_type, name):
@@ -383,6 +385,7 @@ class Generator(object):
 
         for extension in reg.findall('extensions/extension'):
             extname = extension.get('name')
+            cond_extname = "enum_string[enum_string_offsets[i]]"
 
             self.supported_extensions.add(extname)
 
@@ -390,25 +393,21 @@ class Generator(object):
             # or glx, which are separated by '|'
             apis = extension.get('supported').split('|')
             if 'glx' in apis:
-                human_name = 'GLX extension \\"{0}\\"'.format(extname)
-                condition = 'epoxy_conservative_has_glx_extension("{0}")'.format(extname)
+                condition = 'epoxy_conservative_has_glx_extension(provider_name)'
                 loader = 'glXGetProcAddress((const GLubyte *){0})'
-                self.process_require_statements(extension, condition, loader, human_name)
+                self.process_require_statements(extension, condition, loader, extname)
             if 'egl' in apis:
-                human_name = 'EGL extension \\"{0}\\"'.format(extname)
-                condition = 'epoxy_conservative_has_egl_extension("{0}")'.format(extname)
+                condition = 'epoxy_conservative_has_egl_extension(provider_name)'
                 loader = 'eglGetProcAddress({0})'
-                self.process_require_statements(extension, condition, loader, human_name)
+                self.process_require_statements(extension, condition, loader, extname)
             if 'wgl' in apis:
-                human_name = 'WGL extension \\"{0}\\"'.format(extname)
-                condition = 'epoxy_conservative_has_wgl_extension("{0}")'.format(extname)
+                condition = 'epoxy_conservative_has_wgl_extension(provider_name)'
                 loader = 'wglGetProcAddress({0})'
-                self.process_require_statements(extension, condition, loader, human_name)
+                self.process_require_statements(extension, condition, loader, extname)
             if {'gl', 'gles1', 'gles2'}.intersection(apis):
-                human_name = 'GL extension \\"{0}\\"'.format(extname)
-                condition = 'epoxy_conservative_has_gl_extension("{0}")'.format(extname)
+                condition = 'epoxy_conservative_has_gl_extension(provider_name)'
                 loader = 'epoxy_get_proc_address({0})'
-                self.process_require_statements(extension, condition, loader, human_name)
+                self.process_require_statements(extension, condition, loader, extname)
 
     def fixup_bootstrap_function(self, name, loader):
         # We handle glGetString(), glGetIntegerv(), and
@@ -468,6 +467,7 @@ class Generator(object):
                                                                     func.args_decl))
 
     def write_header_header(self, out_file):
+        self.close()
         self.out_file = open(out_file, 'w')
 
         self.outln('/* GL dispatch header.')
@@ -491,6 +491,15 @@ class Generator(object):
             self.outln('#include "epoxy/gl.h"')
             if self.target == "egl":
                 self.outln('#include "EGL/eglplatform.h"')
+                # Account for older eglplatform.h, which doesn't define
+                # the EGL_CAST macro.
+                self.outln('#ifndef EGL_CAST')
+                self.outln('#if defined(__cplusplus)')
+                self.outln('#define EGL_CAST(type, value) (static_cast<type>(value))')
+                self.outln('#else')
+                self.outln('#define EGL_CAST(type, value) ((type) (value))')
+                self.outln('#endif')
+                self.outln('#endif')
         else:
             # Add some ridiculous inttypes.h redefinitions that are
             # from khrplatform.h and not included in the XML.  We
@@ -507,9 +516,17 @@ class Generator(object):
             self.outln('typedef uint32_t khronos_uint32_t;')
             self.outln('typedef uint64_t khronos_uint64_t;')
             self.outln('typedef float khronos_float_t;')
-            self.outln('typedef long khronos_intptr_t;')
-            self.outln('typedef long khronos_ssize_t;')
-            self.outln('typedef unsigned long khronos_usize_t;')
+            self.outln('#ifdef _WIN64')
+            self.outln('typedef signed   long long int khronos_intptr_t;')
+            self.outln('typedef unsigned long long int khronos_uintptr_t;')
+            self.outln('typedef signed   long long int khronos_ssize_t;')
+            self.outln('typedef unsigned long long int khronos_usize_t;')
+            self.outln('#else')
+            self.outln('typedef signed   long int      khronos_intptr_t;')
+            self.outln('typedef unsigned long int      khronos_uintptr_t;')
+            self.outln('typedef signed   long int      khronos_ssize_t;')
+            self.outln('typedef unsigned long int      khronos_usize_t;')
+            self.outln('#endif')
             self.outln('typedef uint64_t khronos_utime_nanoseconds_t;')
             self.outln('typedef int64_t khronos_stime_nanoseconds_t;')
             self.outln('#define KHRONOS_MAX_ENUM 0x7FFFFFFF')
@@ -667,7 +684,7 @@ class Generator(object):
         self.outln('    -1, /* {0}_provider_terminator, unused */'.format(self.target))
         for human_name in sorted_providers:
             enum = self.provider_enum[human_name]
-            self.outln('    {1}, /* {0} */'.format(enum, self.enum_string_offset[human_name]))
+            self.outln('    {1}, /* {0} */'.format(human_name, self.enum_string_offset[human_name]))
         self.outln('};')
         self.outln('')
 
@@ -694,7 +711,9 @@ class Generator(object):
         self.outln('    int i;')
 
         self.outln('    for (i = 0; providers[i] != {0}_provider_terminator; i++) {{'.format(self.target))
+        self.outln('        const char *provider_name = enum_string + enum_string_offsets[providers[i]];')
         self.outln('        switch (providers[i]) {')
+        self.outln('')
 
         for human_name in sorted(self.provider_enum.keys()):
             enum = self.provider_enum[human_name]
@@ -709,6 +728,10 @@ class Generator(object):
         self.outln('    }')
         self.outln('')
 
+        self.outln('    if (epoxy_resolver_failure_handler)')
+        self.outln('        return epoxy_resolver_failure_handler(name);')
+        self.outln('')
+
         # If the function isn't provided by any known extension, print
         # something useful for the poor application developer before
         # aborting.  (In non-epoxy GL usage, the app developer would
@@ -743,6 +766,7 @@ class Generator(object):
         self.outln('')
 
     def write_source(self, f):
+        self.close()
         self.out_file = open(f, 'w')
 
         self.outln('/* GL dispatch code.')
@@ -835,6 +859,12 @@ class Generator(object):
         for func in self.sorted_functions:
             self.write_function_pointer(func)
 
+    def close(self):
+        if self.out_file:
+            self.out_file.close()
+            self.out_file = None
+
+
 argparser = argparse.ArgumentParser(description='Generate GL dispatch wrappers.')
 argparser.add_argument('files', metavar='file.xml', nargs='+', help='GL API XML files to be parsed')
 argparser.add_argument('--outputdir', metavar='dir', required=False, help='Destination directory for files (default to current dir)')
@@ -899,3 +929,5 @@ for f in args.files:
         generator.write_header(os.path.join(includedir, name + '_generated.h'))
     if build_source:
         generator.write_source(os.path.join(srcdir, name + '_generated_dispatch.c'))
+
+    generator.close()
